@@ -248,7 +248,6 @@ get_abundance_sc_wide <- function(.data, transcripts = NULL, all = FALSE) {
 #' @importFrom magrittr "%$%"
 #' @importFrom tidyr pivot_longer
 #' @importFrom tibble as_tibble
-#' @importFrom purrr when
 #' @importFrom purrr map2
 #' @importFrom SummarizedExperiment assays
 #'
@@ -337,8 +336,10 @@ get_abundance_sc_long <- function(.data, transcripts = NULL, all = FALSE,
     Reduce(function(...) left_join(..., by = c(f_(.data)$name, "cell")), .)
 }
 
+
 #' @importFrom methods .hasSlot
 #' @importFrom S4Vectors DataFrame
+#' @importFrom stats setNames
 #' @importFrom SummarizedExperiment colData
 #' @importFrom SummarizedExperiment rowData
 #' @importFrom SummarizedExperiment colData<-
@@ -357,70 +358,8 @@ update_SE_from_tibble <- function(.data_mutated, se, column_belonging = NULL) {
   # Comply to CRAN notes 
   . <- NULL 
   
-  # Get the colnames of samples and feature datasets
-  colnames_col <- 
-    colnames(colData(se)) %>% 
-    c(s_(se)$name) %>%
-    
-    # Forcefully add the column I know the source. This is useful in nesting 
-    # where a unique value cannot be linked to sample or feature
-    c(names(column_belonging[column_belonging == s_(se)$name]))
-  
-  colnames_row <- se %>%
-    when(
-      .hasSlot(., "rowData") | .hasSlot(., "elementMetadata") ~ colnames(rowData(.)), 
-      TRUE ~ c()
-    ) %>% 
-    c(f_(se)$name) %>%
-    
-    # Forcefully add the column I know the source. This is useful in nesting 
-    # where a unique value cannot be linked to sample or feature
-    c(names(column_belonging[column_belonging == f_(se)$name]))
-  
-  special_columns <- get_special_columns(
-    # Decrease the size of the dataset
-    se[1:min(100, nrow(se)), min(1, ncol(se)):min(20, ncol(se))]
-  ) 
-  
-  # This is used if I have one column with one value that can be mapped to rows and columns
-  new_colnames_col = c()
-  
-  # This condiction is because if I don't have any samples, the new column 
-  # could be mapped to samples and return NA in the final SE
-  if(ncol(se) > 0)
-    new_colnames_col <- 
-    .data_mutated %>%
-    select_if(!colnames(.) %in% setdiff(colnames_col, s_(se)$name)) %>% 
-    
-    # Eliminate special columns that are read only. Assays
-    select_if(!colnames(.) %in% special_columns) %>%
-    select_if(!colnames(.) %in% colnames_row) %>%
-    # Replace for subset
-    select(!!s_(se)$symbol, get_subset_columns(., !!s_(se)$symbol)) %>% 
-    colnames()
-  
-  col_data <-
-    .data_mutated %>%
-    
-    select(c(colnames_col, new_colnames_col)) %>%
-    
-    
-    # Filter Sample is NA from SE that have 0 samples
-    filter(!is.na(!!s_(se)$symbol)) 
-  
-  # This works even if I have 0 samples
-  duplicated_samples = col_data |> pull(!!s_(se)$symbol) %>% duplicated() 
-  if(duplicated_samples |> which() |>  length() > 0)
-    # Make fast distinct()
-    col_data = col_data |> filter(!duplicated_samples)
-  
-  col_data = 
-    col_data |> 
-    
-    # In case unitary SE subset does not work
-    data.frame(row.names = pull(col_data, !!s_(se)$symbol), check.names = FALSE) %>%
-    select(-!!s_(se)$symbol) %>%
-    DataFrame(check.names = FALSE)
+  # Extract column data
+  col_data <- extract_col_data(.data_mutated, se, column_belonging)
   
   # This to avoid the mismatch between missing column names for counts 
   # and numerical row names for colData
@@ -432,26 +371,8 @@ update_SE_from_tibble <- function(.data_mutated, se, column_belonging = NULL) {
       ~ (.)
     )
   
-  row_data <-
-    .data_mutated %>%
-    
-    # Eliminate special columns that are read only 
-    select_if(!colnames(.) %in% special_columns) %>%
-    
-    #eliminate sample columns directly
-    select_if(!colnames(.) %in% c(s_(se)$name, colnames(col_data))) %>%
-    
-    # select(one_of(colnames(rowData(se))))
-    # Replace for subset
-    select(!!f_(se)$symbol,  get_subset_columns(., !!f_(se)$symbol) ) %>%
-    
-    # Make fast distinct()
-    filter(pull(., !!f_(se)$symbol) %>% duplicated() %>% not()) %>% 
-    
-    # In case unitary SE subset does not work because all same
-    data.frame(row.names = pull(.,f_(se)$symbol), check.names = FALSE) %>%
-    select(-!!f_(se)$symbol) %>%
-    DataFrame(check.names = FALSE)
+  # Extract row data
+  row_data <- extract_row_data(.data_mutated, se, col_data = col_data)
   
   # This to avoid the mismatch between missing row names for counts 
   # and numerical row names for rowData
@@ -470,17 +391,12 @@ update_SE_from_tibble <- function(.data_mutated, se, column_belonging = NULL) {
   colData(se) <- col_data
   rowData(se) <- row_data
   
-  # Count-like data that is NOT in the assay slot already 
-  colnames_assay <-
-    colnames(.data_mutated) %>% 
-    setdiff(c(f_(se)$name, s_(se)$name, colnames(as.data.frame(head(rowRanges(se), 1))) )) %>%
-    setdiff(colnames(col_data)) %>% 
-    setdiff(colnames(row_data)) %>%
-    setdiff(assays(se) %>% names)
+  # Extract assay column names
+  colnames_assay <- extract_assay_colnames(.data_mutated, se, col_data, row_data)
   
   if (length(colnames_assay) > 0)
-    assays(se, withDimnames = FALSE) <- 
-    c(assays(se, withDimnames = FALSE),
+    assays(se, withDimnames = FALSE) <-
+    assays(se, withDimnames = FALSE) %>% c(
       .data_mutated %>% 
         
         # Select assays column
@@ -521,23 +437,82 @@ update_SE_from_tibble <- function(.data_mutated, se, column_belonging = NULL) {
   se
 }
 #' @importFrom methods is
-slice_optimised <- function(.data, ..., .preserve=FALSE) {
+#' @keywords internal
+#' @param slice_args List of arguments passed to slice
+#' @param .data The data object to check for grouping
+#' @param .by The .by argument from slice
+#' @return Logical indicating if this is an ungrouped range slice
+#' @noRd
+is_range_slice_ungrouped_detected <- function(.data, slice_args, .by = NULL) {
+  # First check if this is a range slice
+  is_range_slice <- any(sapply(slice_args, function(x) {
+    if (is.numeric(x)) {
+      length(x) > 1
+    } else {
+      FALSE
+    }
+  }))
+  
+  # If not a range slice, return FALSE (no error needed)
+  if (!is_range_slice) {
+    return(FALSE)
+  }
+  
+  # If it is a range slice, check if the data is grouped
+  # Check both .by parameter and if the data itself is grouped
+  is_grouped_by_param <- !quo_is_null(.by)
+  is_grouped_by_data <- inherits(.data, "grouped_df") || 
+    (inherits(.data, "SummarizedExperiment") && !is.null(attr(.data, "groups")))
+  
+  is_grouped <- is_grouped_by_param || is_grouped_by_data
+  
+  # Return TRUE if ungrouped (should throw error)
+  !is_grouped
+}
+
+slice_optimised <- function(.data, ..., .by = NULL, .preserve = FALSE) {
   
   . <- NULL
   
+  .by <- enquo(.by)
+
+  # For slice operations that might select multiple rows, we need to handle the case where
+  # the result cannot maintain the SummarizedExperiment structure
+  # In this case, we should return a tibble with the equivalent data
+  # Check if this is an ungrouped range slice
+  slice_args <- list(...)
+  if (is_range_slice_ungrouped_detected(.data, slice_args, .by)) {
+    # For range slices on ungrouped data, throw an error with a helpful message
+    stop("tidySummarizedExperiment says: slice using a range doesn't work on ungrouped data. Please use .by parameter or convert to tibble with as_tibble() before using slice with ranges.")
+  }
+  
+  
+  # For single row slices, use the original optimized approach
   # This simulated tibble only gets samples and features so we know those that have been completely omitted already
   # In order to save time for the as_tibble conversion
   simulated_slice = 
-    simulate_feature_sample_from_tibble(.data) %>% 
-    dplyr::slice(..., .preserve = .preserve)
+    simulate_feature_sample_from_tibble(.data, !!.by) %>% 
+    dplyr::slice(..., .by = !!.by, .preserve = .preserve)
   
+  # Remove .by column only if it's not a special column
+  if (!rlang::quo_is_null(.by) && !quo_name(.by) %in% c(f_(.data)$name, s_(.data)$name)) {
+    simulated_slice <- simulated_slice %>% select(-!!.by)
+  }
+ 
   .data = 
     .data %>%
     
     # Subset the object for samples and features present in the simulated data
-    .[rownames(.) %in% simulated_slice[,f_(.)$name], 
-      colnames(.) %in% simulated_slice[,s_(.)$name]] %>% 
-    inner_join(simulated_slice, by = c(f_(.)$name, s_(.)$name)) 
+    .[rownames(.data) %in% simulated_slice[,f_(.data)$name][[1]], 
+      colnames(.data) %in% simulated_slice[,s_(.data)$name][[1]]
+      ] 
+  .data = .data %>% 
+    inner_join(simulated_slice, by = c(f_(.data)$name, s_(.data)$name)) 
+  
+  # If the result is already a tibble (due to mixed scope join), return it directly
+  if (.data %>% is("tbl")) {
+    return(.data)
+  }
   
   # If order do not match with the one proposed by slice convert to tibble
   if (.data %>% is("tbl") %>% not()) {
@@ -855,7 +830,6 @@ quo_names <- function(v) {
     unlist()
 }
 
-#' @importFrom purrr when
 #' @importFrom dplyr select
 #' @importFrom rlang expr
 #' @importFrom tidyselect eval_select
@@ -1060,128 +1034,6 @@ choose_name_if_present <- function(x) {
   columns_query
 }
 
-#' @importFrom purrr when
-join_efficient_for_SE <- function(x, y, by = NULL, copy = FALSE, 
-                                  suffix = c(".x", ".y"), join_function, 
-                                  force_tibble_route = FALSE,
-                                  ...) {
-  
-  # Comply to CRAN notes 
-  . <- NULL 
-  
-  # Deprecation of special column names
-  if (is_sample_feature_deprecated_used(x, when(by, !is.null(.) ~ by, ~ colnames(y)))) {
-    x <- ping_old_special_column_into_metadata(x)
-  }
-  
-  # Get the colnames of samples and feature datasets
-  colnames_col <- get_colnames_col(x)
-  colnames_row <- get_rownames_col(x)
-  
-  # See if join done by sample, feature or both
-  columns_query <- by %>% when(
-    !is.null(.) ~ choose_name_if_present(.), 
-    ~ colnames(y) %>% intersect(c(colnames_col, colnames_row))
-  )
-  
-  if (
-    # Complex join that it is not efficient yet
-    (any(columns_query %in% colnames_row) & any(columns_query %in% colnames_col)) |
-    
-    # If join is with something else, the inefficient generic solution might work, 
-    # or delegate the proper error downstream
-    (!any(columns_query %in% colnames_row) & !any(columns_query %in% colnames_col)) |
-    
-    # Needed for internal recurrence if outcome is not valid
-    force_tibble_route) {
-    
-    # If I have a big dataset
-    if (ncol(x) > 100) message("tidySummarizedExperiment says: if you are joining a dataframe both sample-wise and feature-wise, for efficiency (until further development), it is better to separate your joins and join datasets sample-wise OR feature-wise.")
-    
-    x %>%
-      as_tibble(skip_GRanges = TRUE) %>%
-      join_function(y, by = by, copy = copy, suffix = suffix, ...) %>%
-      when(
-        
-        # If duplicated sample-feature pair returns tibble
-        !is_not_duplicated(., x) | !is_rectangular(., x) ~ {
-          message(duplicated_cell_names)
-          message(data_frame_returned_message)
-          (.)
-        },
-        
-        # Otherwise return updated tidySummarizedExperiment
-        ~ update_SE_from_tibble(., x)
-      )
-    
-  }
-  
-  # Join only feature-wise
-  else if (any(columns_query %in% colnames_row) & !any(columns_query %in% colnames_col)) {
-    
-    row_data_tibble <-  
-      rowData(x) %>% 
-      as_tibble(rownames = f_(x)$name) %>%  
-      join_function(y, by = by, copy = copy, suffix = suffix, ...) 
-    
-    # Check if the result is not SE then take the tibble route
-    if (
-      is.na(pull(row_data_tibble, !!f_(x)$symbol)) %>% any | 
-      duplicated(pull(row_data_tibble, !!f_(x)$symbol)) %>% any |
-      pull(row_data_tibble, !!f_(x)$symbol) %>% setdiff(rownames(colData(x))) %>% length() %>% gt(0)
-    ) return(join_efficient_for_SE(x, y, by = by, copy = copy, suffix = suffix, 
-                                   join_function, force_tibble_route = TRUE, ...))
-    
-    row_data <- 
-      row_data_tibble %>% 
-      data.frame(row.names = pull(., !!f_(x)$symbol)) %>%
-      select(-!!f_(x)$symbol) %>%
-      DataFrame()
-    
-    # Subset in case of an inner join, or a right join
-    x <- x[rownames(row_data),]  
-    
-    # Tranfer annotation
-    rowData(x) <- row_data
-    
-    # Return
-    x
-  }
-  
-  # Join only sample-wise
-  else if (any(columns_query %in% colnames_col) & !any(columns_query %in% colnames_row)) {
-    
-    col_data_tibble <- 
-      colData(x) %>% 
-      as_tibble(rownames = s_(x)$name) %>%  
-      join_function(y, by = by, copy = copy, suffix = suffix, ...)
-    
-    # Check if the result is not SE then take the tibble route
-    if (
-      is.na(pull(col_data_tibble, !!s_(x)$symbol)) %>% any | 
-      duplicated(pull(col_data_tibble, !!s_(x)$symbol)) %>% any |
-      pull(col_data_tibble, !!s_(x)$symbol) %>% setdiff(rownames(colData(x))) %>% length() %>% gt(0)
-    ) return(join_efficient_for_SE(x, y, by = by, copy = copy, suffix = suffix, 
-                                   join_function, force_tibble_route = TRUE, ...))
-    
-    col_data <- 
-      col_data_tibble %>% 
-      data.frame(row.names = pull(., !!s_(x)$symbol)) %>%
-      select(-!!s_(x)$symbol) %>%
-      DataFrame()
-    
-    # Subset in case of an inner join, or a right join
-    x <- x[,rownames(col_data)]  
-    
-    # Transfer annotation
-    colData(x) <- col_data
-    
-    # Return
-    x
-  }
-  
-  else stop("tidySummarizedExperiment says: ERROR FOR DEVELOPERS: this option should not exist. In join utility.")
-}
 
 get_ellipse_colnames <- function(...) {
   (enquos(..., .ignore_empty = "all") %>% map(~ quo_name(.x)) %>% unlist)
@@ -1255,13 +1107,103 @@ get_special_column_name_symbol <- function(name) {
 # This function produce artificially the feature ans sample column, 
 # to make optimisation before as_tibble is called
 # for big datasets
-simulate_feature_sample_from_tibble <- function(.data) {
+simulate_feature_sample_from_tibble <- function(.data, ...) {
 
     . <- NULL
     r <- rownames(.data) %>% .[rep(1:length(.), ncol(.data) )]
     c <- colnames(.data) %>% .[rep(1:length(.), each = nrow(.data) )]
     
-    tibble(!!f_(.data)$symbol := r,  !!s_(.data)$symbol := c)
+    # Create base tibble with feature and sample columns
+    result_tibble <- tibble(!!f_(.data)$symbol := r,  !!s_(.data)$symbol := c)
+    
+    # BELOW IS THE MORE COMPLEX GROUP CASE
+    # THAT SLOWS DOWN 10x
+    # IN THE FUTURE THIS COULD BE OPTIMISED
+    
+    # Store original dimensions for use in rowData/colData processing
+    n_features_orig <- nrow(.data)
+    n_samples_orig <- ncol(.data)
+    feature_names_orig <- rownames(.data)
+    sample_names_orig <- colnames(.data)
+    
+    # Store column names to avoid issues with .data references
+    feature_col_name <- f_(.data)$name
+    sample_col_name <- s_(.data)$name
+    feature_col_symbol <- f_(.data)$symbol
+    sample_col_symbol <- s_(.data)$symbol
+    
+    # Check if ... parameters reference columns in rowData or colData
+    # and attach only those columns to the output
+    
+    # Extract column names from ... parameters
+    slice_colnames <- get_ellipse_colnames(...)
+    
+    # Filter out any non-character elements (like quosures)
+    slice_colnames <- slice_colnames[is.character(slice_colnames)]
+    
+    # Only proceed if there are slice parameters
+    if (length(slice_colnames) > 0) {
+        # Get available rowData and colData column names
+        rowdata_colnames <- if (.hasSlot(.data, "rowData") | .hasSlot(.data, "elementMetadata")) {
+            colnames(rowData(.data))
+        } else {
+            character(0)
+        }
+        
+        coldata_colnames <- colnames(colData(.data))
+        
+        # Check which slice columns are in rowData
+        rowdata_matches <- intersect(slice_colnames, rowdata_colnames)
+        
+        # Check which slice columns are in colData  
+        coldata_matches <- intersect(slice_colnames, coldata_colnames)
+        
+        # Add rowData columns if they are referenced in ...
+        if (length(rowdata_matches) > 0) {
+            # Create rowData tibble with repeated rows for each sample
+            rowdata_df <- as.data.frame(rowData(.data)[, rowdata_matches, drop = FALSE])
+            rowdata_df[[feature_col_name]] <- rownames(rowData(.data))
+            
+            # Create expanded tibble more efficiently
+            rowdata_tibble <- tibble(
+                !!feature_col_symbol := rep(rowdata_df[[feature_col_name]], each = n_samples_orig),
+                !!sample_col_symbol := rep(sample_names_orig, n_features_orig)
+            )
+            
+            # Add the rowData columns
+            for (col in rowdata_matches) {
+                rowdata_tibble[[col]] <- rep(rowdata_df[[col]], each = n_samples_orig)
+            }
+            
+            # Join with result tibble
+            result_tibble <- result_tibble %>%
+                left_join(rowdata_tibble, by = c(feature_col_name, sample_col_name))
+        }
+        
+        # Add colData columns if they are referenced in ...
+        if (length(coldata_matches) > 0) {
+            # Create colData tibble with repeated columns for each feature
+            coldata_df <- as.data.frame(colData(.data)[, coldata_matches, drop = FALSE])
+            coldata_df[[sample_col_name]] <- rownames(colData(.data))
+            
+            # Create expanded tibble more efficiently
+            coldata_tibble <- tibble(
+                !!feature_col_symbol := rep(feature_names_orig, each = n_samples_orig),
+                !!sample_col_symbol := rep(coldata_df[[sample_col_name]], n_features_orig)
+            )
+            
+            # Add the colData columns
+            for (col in coldata_matches) {
+                coldata_tibble[[col]] <- rep(coldata_df[[col]], n_features_orig)
+            }
+            
+            # Join with result tibble
+            result_tibble <- result_tibble %>%
+                left_join(coldata_tibble, by = c(feature_col_name, sample_col_name))
+        }
+    }
+    
+    return(result_tibble)
 }
 
 feature__ <- get_special_column_name_symbol(".feature")
@@ -1463,3 +1405,20 @@ reduce_rbind_se <- function(se_list){
   # Return
   se
 }
+
+
+# Helper function to convert tibble back to DataFrame with rownames
+DataFrame_rownames <- function(data, rownames_col = "rowname__") {
+  if (rownames_col %in% colnames(data)) {
+    # Extract rownames and remove the rownames column
+    row_names <- data[[rownames_col]]
+    data_without_rownames <- data[, colnames(data) != rownames_col, drop = FALSE]
+    
+    # Convert to DataFrame with rownames
+    result <- DataFrame(data_without_rownames, row.names = row_names)
+    return(result)
+  } else {
+    # No rownames column, just convert to DataFrame
+    return(as(data, "DataFrame"))
+  }
+}
\ No newline at end of file
